@@ -15,15 +15,15 @@
 #include "change-db-cb.h"
 #include "new-db-cb.h"
 #include "../common/secret-schema.h"
+#include "../common/macros.h"
 #include "change-pwd-cb.h"
 #include "settings-cb.h"
 #include "shortcuts-cb.h"
 #include "webcam-add-cb.h"
 #include "manual-add-cb.h"
-#include "edit-row-cb.h"
-#include "show-qr-cb.h"
 #include "dbinfo-cb.h"
 #include "change-file-cb.h"
+#include "change-db-sec.h"
 
 #ifndef IS_FLATPAK
 static gchar     *get_db_path               (AppData            *app_data);
@@ -57,9 +57,6 @@ static void       toggle_button_cb          (GtkWidget          *main_window,
 static void       reorder_rows_cb           (GtkToggleButton *btn,
                                              gpointer         user_data);
 
-static void       del_data_cb               (GtkToggleButton    *btn,
-                                             gpointer            user_data);
-
 static void       save_sort_order           (GtkTreeView        *tree_view);
 
 static void       save_window_size          (gint                width,
@@ -83,7 +80,7 @@ static void       set_open_db_action        (GtkWidget          *btn,
 
 void
 activate (GtkApplication    *app,
-          gpointer           user_data __attribute__((unused)))
+          gpointer           user_data UNUSED)
 {
     gint32 max_file_size = get_max_file_size_from_memlock ();
 
@@ -212,7 +209,7 @@ activate (GtkApplication    *app,
 
     GError *err = NULL;
     load_db (app_data->db_data, &err);
-    if (err != NULL && !g_error_matches (err, missing_file_gquark (), MISSING_FILE_CODE)) {
+    if (err != NULL && !g_error_matches (err, missing_file_gquark (), MISSING_FILE_ERRCODE)) {
         show_message_dialog (app_data->main_window, err->message, GTK_MESSAGE_ERROR);
         gcry_free (app_data->db_data->key);
         if (g_error_matches (err, memlock_error_gquark (), MEMLOCK_ERRCODE)) {
@@ -230,7 +227,7 @@ activate (GtkApplication    *app,
         secret_password_store (OTPCLIENT_SCHEMA, SECRET_COLLECTION_DEFAULT, "main_pwd", app_data->db_data->key, NULL, on_password_stored, NULL, "string", "main_pwd", NULL);
     }
 
-    if (g_error_matches (err, missing_file_gquark(), MISSING_FILE_CODE)) {
+    if (g_error_matches (err, missing_file_gquark(), MISSING_FILE_ERRCODE)) {
         const gchar *msg = _("This is the first time you run OTPClient, so you need to <b>add</b> or <b>import</b> some tokens.\n"
         "- to <b>add</b> tokens, please click the + button on the <b>top left</b>.\n"
         "- to <b>import</b> existing tokens, please click the menu button <b>on the top right</b>.\n"
@@ -259,9 +256,6 @@ activate (GtkApplication    *app,
     g_signal_connect (reorder_toggle_btn, "toggled", G_CALLBACK(reorder_rows_cb), app_data);
     g_signal_connect (app_data->main_window, "key_press_event", G_CALLBACK(key_pressed_cb), NULL);
 
-    GtkToggleButton *del_toggle_btn = GTK_TOGGLE_BUTTON(gtk_builder_get_object (app_data->builder, "del_toggle_btn_id"));
-    g_signal_connect (app_data->main_window, "toggle-delete-button", G_CALLBACK(toggle_button_cb), del_toggle_btn);
-    g_signal_connect (del_toggle_btn, "toggled", G_CALLBACK(del_data_cb), app_data);
     g_signal_connect (app_data->main_window, "key_press_event", G_CALLBACK(key_pressed_cb), NULL);
 
     g_signal_connect (app_data->main_window, "destroy", G_CALLBACK(destroy_cb), app_data);
@@ -314,7 +308,7 @@ show_memlock_warn_dialog (gint32      max_file_size,
 static gboolean
 key_pressed_cb (GtkWidget   *window,
                 GdkEventKey *event_key,
-                gpointer     user_data __attribute__((unused)))
+                gpointer     user_data UNUSED)
 {
     switch (event_key->keyval) {
         case GDK_KEY_q:
@@ -473,8 +467,7 @@ create_main_window (gint     width,
             { .name = "create_newdb", .activate = new_db_cb },
             { .name = "change_db", .activate = change_db_cb },
             { .name = "change_pwd", .activate = change_password_cb },
-            { .name = "edit_row", .activate = edit_row_cb },
-            { .name = "show_qr", .activate = show_qr_cb },
+            { .name = "change_db_sec", .activate = change_db_sec_cb },
             { .name = "settings", .activate = settings_dialog_cb },
             { .name = "shortcuts", .activate = shortcuts_window_cb },
             { .name = "dbinfo", .activate = dbinfo_cb },
@@ -495,20 +488,21 @@ create_main_window (gint     width,
     g_action_map_add_action_entries (G_ACTION_MAP(settings_actions), settings_menu_entries, G_N_ELEMENTS (settings_menu_entries), app_data);
     gtk_widget_insert_action_group (settings_popover, "settings_menu", settings_actions);
 
-    GActionGroup *import_actions = (GActionGroup *)g_simple_action_group_new ();
-    g_action_map_add_action_entries (G_ACTION_MAP(import_actions), import_menu_entries, G_N_ELEMENTS (import_menu_entries), app_data);
-    gtk_widget_insert_action_group (settings_popover, "import_menu", import_actions);
-
     GActionGroup *export_actions = (GActionGroup *)g_simple_action_group_new ();
     g_action_map_add_action_entries (G_ACTION_MAP(export_actions), export_menu_entries, G_N_ELEMENTS (export_menu_entries), app_data);
     gtk_widget_insert_action_group (settings_popover, "export_menu", export_actions);
 
     GtkWidget *add_popover = GTK_WIDGET(gtk_builder_get_object (app_data->add_popover_builder, "add_pop_id"));
     gtk_menu_button_set_popover (GTK_MENU_BUTTON(gtk_builder_get_object (app_data->builder, "add_btn_main_id")), add_popover);
+
     GActionGroup *add_actions = (GActionGroup *)g_simple_action_group_new ();
     g_action_map_add_action_entries (G_ACTION_MAP(add_actions), add_menu_entries, G_N_ELEMENTS (add_menu_entries), app_data);
     gtk_widget_insert_action_group (add_popover, "add_menu", add_actions);
 
+    GActionGroup *import_actions = (GActionGroup *)g_simple_action_group_new ();
+    g_action_map_add_action_entries (G_ACTION_MAP(import_actions), import_menu_entries, G_N_ELEMENTS (import_menu_entries), app_data);
+    gtk_widget_insert_action_group (add_popover, "import_menu", import_actions);
+
     gtk_popover_set_constrain_to (GTK_POPOVER(add_popover), GTK_POPOVER_CONSTRAINT_NONE);
     gtk_popover_set_constrain_to (GTK_POPOVER(settings_popover), GTK_POPOVER_CONSTRAINT_NONE);
 }
@@ -583,7 +577,7 @@ get_db_path (AppData *app_data)
 
 
 static void
-toggle_button_cb (GtkWidget *main_window __attribute__((unused)),
+toggle_button_cb (GtkWidget *main_window UNUSED,
                   gpointer   user_data)
 {
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(user_data), !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(user_data)));
@@ -594,12 +588,11 @@ static void
 reorder_rows_cb (GtkToggleButton *btn,
                  gpointer         user_data)
 {
-    AppData *app_data = (AppData *)user_data;
+    CAST_USER_DATA(AppData, app_data, user_data);
     gboolean is_btn_active = gtk_toggle_button_get_active (btn);
     gtk_tree_view_set_reorderable (GTK_TREE_VIEW(app_data->tree_view), is_btn_active);
     app_data->is_reorder_active = is_btn_active;
     gtk_widget_set_sensitive (GTK_WIDGET(gtk_builder_get_object (app_data->builder, "add_btn_main_id")), !is_btn_active);
-    gtk_widget_set_sensitive (GTK_WIDGET(gtk_builder_get_object (app_data->builder, "del_toggle_btn_id")), !is_btn_active);
 
     if (is_btn_active == FALSE) {
         // reordering has been disabled, so now we have to reorder and update the database itself
@@ -608,55 +601,10 @@ reorder_rows_cb (GtkToggleButton *btn,
 }
 
 
-static void
-del_data_cb (GtkToggleButton *btn,
-             gpointer         user_data)
-{
-    AppData *app_data = (AppData *)user_data;
-
-    GtkStyleContext *gsc_btn = gtk_widget_get_style_context (GTK_WIDGET(btn));
-    GtkStyleContext *gsc_tv = gtk_widget_get_style_context (GTK_WIDGET(app_data->tree_view));
-
-    GtkTreeSelection *tree_selection = gtk_tree_view_get_selection (app_data->tree_view);
-
-    if (gtk_toggle_button_get_active (btn)) {
-        app_data->delbtn_css_provider = gtk_css_provider_new ();
-        app_data->tv_css_provider = gtk_css_provider_new ();
-
-        gtk_css_provider_load_from_data (app_data->delbtn_css_provider, "#delbtn { background: #970000; }", -1, NULL);
-        gtk_css_provider_load_from_data (app_data->tv_css_provider, "#tv { background: #970000; }", -1, NULL);
-
-        gtk_style_context_add_provider (gsc_btn, GTK_STYLE_PROVIDER(app_data->delbtn_css_provider), GTK_STYLE_PROVIDER_PRIORITY_USER);
-        gtk_style_context_add_provider (gsc_tv, GTK_STYLE_PROVIDER(app_data->tv_css_provider), GTK_STYLE_PROVIDER_PRIORITY_USER);
-
-        const gchar *msg = _("You just entered the deletion mode. You can now click on the row(s) you'd like to delete.\n"
-            "Please note that once a row has been deleted, <b>it's impossible to recover the associated data.</b>");
-
-        if (get_confirmation_from_dialog (app_data->main_window, msg)) {
-            g_signal_handlers_disconnect_by_func (app_data->tree_view, row_selected_cb, app_data);
-            // the following function emits the "changed" signal
-            gtk_tree_selection_unselect_all (tree_selection);
-            // clear all active otps before proceeding to the deletion phase
-            g_signal_emit_by_name (app_data->tree_view, "hide-all-otps");
-            g_signal_connect (app_data->tree_view, "row-activated", G_CALLBACK(delete_rows_cb), app_data);
-        } else {
-            gtk_toggle_button_set_active (btn, FALSE);
-        }
-    } else {
-        gtk_style_context_remove_provider (gsc_btn, GTK_STYLE_PROVIDER(app_data->delbtn_css_provider));
-        gtk_style_context_remove_provider (gsc_tv, GTK_STYLE_PROVIDER(app_data->tv_css_provider));
-        g_object_unref (app_data->delbtn_css_provider);
-        g_object_unref (app_data->tv_css_provider);
-        g_signal_handlers_disconnect_by_func (app_data->tree_view, delete_rows_cb, app_data);
-        g_signal_connect (app_data->tree_view, "row-activated", G_CALLBACK(row_selected_cb), app_data);
-    }
-}
-
-
 static void
 get_window_size_cb (GtkWidget      *window,
-                    GtkAllocation  *allocation __attribute__((unused)),
-                    gpointer        user_data  __attribute__((unused)))
+                    GtkAllocation  *allocation UNUSED,
+                    gpointer        user_data UNUSED)
 {
     gint w, h;
     gtk_window_get_size (GTK_WINDOW(window), &w, &h);
@@ -669,7 +617,7 @@ void
 destroy_cb (GtkWidget   *window,
             gpointer     user_data)
 {
-    AppData *app_data = (AppData *)user_data;
+    CAST_USER_DATA(AppData, app_data, user_data);
     save_sort_order (app_data->tree_view);
     g_source_remove (app_data->source_id);
     g_source_remove (app_data->source_id_last_activity);
@@ -681,7 +629,7 @@ destroy_cb (GtkWidget   *window,
     gcry_free (app_data->db_data->key);
     g_free (app_data->db_data->db_path);
     g_slist_free_full (app_data->db_data->objects_hash, g_free);
-    json_decref (app_data->db_data->json_data);
+    json_decref (app_data->db_data->in_memory_json_data);
     g_free (app_data->db_data);
     gtk_clipboard_clear (app_data->clipboard);
     g_application_withdraw_notification (G_APPLICATION(gtk_window_get_application (GTK_WINDOW(app_data->main_window))), NOTIFICATION_ID);
@@ -757,7 +705,7 @@ static void
 set_open_db_action (GtkWidget *btn,
                     gpointer   user_data)
 {
-    AppData *app_data = (AppData *)user_data;
+    CAST_USER_DATA(AppData, app_data, user_data);
     app_data->open_db_file_action = g_strcmp0 (gtk_widget_get_name (btn), "diag_rc_restoredb_btn") == 0 ? GTK_FILE_CHOOSER_ACTION_OPEN : GTK_FILE_CHOOSER_ACTION_SAVE;
     gtk_dialog_response (GTK_DIALOG(app_data->diag_rcdb), GTK_RESPONSE_OK);
 }
